package com.idiot.operationbackend.controller.interactive;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.idiot.operationbackend.entity.*;
import com.idiot.operationbackend.service.facade.*;
import com.idiot.operationbackend.support.JsonResult;
import com.idiot.operationbackend.util.JwtTokenUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 二维码
 * @author wang xiao
 * @date Created in 18:03 2020/9/21
 */
@RestController
@RequestMapping("/qr")
@Api(value = "QrCodeController", tags ="二维码")
public class QrCodeController {


    private final Logger logger = LoggerFactory.getLogger(QrCodeController.class);

    @Autowired
    private AccountService accountService;

    @Autowired
    private QrCodeService qrCodeService;


    @Autowired
    private WeChatService weChatService;

    @GetMapping
    @ApiOperation(value = "查询二维码列表")
    public ResponseEntity<JsonResult<Page<QrCode>>> qrCodePage (@RequestHeader String token,int page) {
        String userId = JwtTokenUtil.getUserId(token);
        logger.info("用户:{}查询二维码列表",userId);
        List<Account> accounts = accountService.queryAccountByUserId(userId);
        List<String> accIds = accounts.stream().map(Account::getId).collect(Collectors.toList());
        return ResponseEntity.ok(JsonResult.success(qrCodeService.pageQrCode(page,accIds)));
    }


    @GetMapping("/{id}")
    @ApiOperation(value = "查询二维码详情")
    public ResponseEntity<JsonResult<QrCode>> qrCode (@RequestHeader String token,
                                                      @PathVariable String id) {
        String userId = JwtTokenUtil.getUserId(token);
        logger.info("用户:{}查询二维码:{}详情",userId,id);
        QrCode qrCode = qrCodeService.getById(id);
        return ResponseEntity.ok(JsonResult.success(qrCode));
    }

    @PostMapping
    @ApiOperation(value = "新增二维码")
    public ResponseEntity<JsonResult<Boolean>> addQrCode (@RequestHeader String token,
                                                          @RequestBody @Valid  QrCode qrCode) {
        String userId = JwtTokenUtil.getUserId(token);
        logger.info("用户:{}新增二维码",userId);
        String accountId = qrCode.getAccountId();
        Account account = accountService.getById(accountId);
        qrCode.setNikeName(account.getNickName());
        qrCode.setHeadImage(account.getHeadImage());
        qrCodeService.save(qrCode);
        String wxJson = weChatService.createQrCode(qrCode,accountId);
        boolean upResult = qrCodeService.upQrCode(qrCode,wxJson);
        return ResponseEntity.ok(JsonResult.success(upResult));
    }



}
